import sys
import pydot
from numarray import *
import numarray.linear_algebra as la

def nodes(g):
	"""Returns the _actual_ nodes of the graph g.
	
	That is, throws out the 'graph', 'edge', and 'node' nodes
	returned by pydot's get_node_list() function."""
	ns0 = g.get_node_list()
	ns = []
	for n in ns0:
		if n.name!='graph' and n.name!='edge' and n.name!='node':
			ns.append(n)
	return ns

def index(ns, name):
	"""Returns the index of the node named name in the node list ns."""
	return [n.name for n in ns].index(name)

def make_color_dictionary(es, cdict0={}):
	"""Returns a dictionary associating colors with weights, based on
	the inputted edges es. The bindings passed in cdict0 take precedence.
	
	If a key is not already bound in cdict0, bindings are determined from
	edges with specified weight values, the first of which takes
	precedence for a given color. Note that even colorless edges will
	result in a binding if their weight is specified, i.e. they will add
	the 'default' binding for colorless edges."""
	cdict = cdict0.copy()
	for e in es:
		if e.weight!=None and (e.color not in cdict):
			cdict[e.color] = float(e.weight)
	return cdict

def currents(g,ns,cdict={},r_weights=False):
	"""Returns a matrix representing the currents through the nodes ns
	of a graph g, assuming conductances are given as the edge weights,
	or as colors whose conductances can be looked up in cdict. If
	r_weights is true, the values in cdict and the edge weights are
	treated as resistances instead of conductances.
	
	ns will most often be the output of nodes(g), but one can pass a
	subset of that list if so desired.
	The format of the output matrix, call it cur, is such that for
	each pair of nodes ns[i] and ns[j], the component of the current
	going into ns[i] from ns[j] is cur[i,j] times the voltage at ns[j].
	
	Essentially, this function returns the matrix representing the
	equations found by Kirchoff's Current Law."""
	n = len(ns)
	curs = zeros((n,n),type=Float32)
	for i in range(n):
		for j in range(n):
			e1 = g.get_edge(ns[i].name,ns[j].name)
			e2 = g.get_edge(ns[j].name,ns[i].name)
			if e1:
				if e1.weight==None:
					cond = cdict[e1.color]
				else:
					cond = float(e1.weight)
				if r_weights:
					cond = 1.0 / cond
			elif e2:
				if e2.weight==None:
					cond = cdict[e2.color]
				else:
					cond = float(e2.weight)
				if r_weights:
					cond = 1.0 / cond
			else:
				cond = 0
			curs[i,j] -= cond
			curs[i,i] += cond
	return curs

def conductance(curs,a,b):
	"""Given a matrix of currents, such as one returned by currents(), this
	function finds the equivalent conductance between the two nodes indexed
	by a and b.
	
	Note that the graph used to create the currents matrix must be connected,
	or at least any disjoint nodes must be left out, since otherwise a singular
	matrix will of course result (since any voltages will be possible at the
	disjoint nodes).
	
	This is accomplished by 'applying' a voltage of 1 at a and 0 at b, and
	solving for the resulting current by solving the equations embodied in
	curs."""
	if a==b:
		raise ValueError('infinite conductance!')
	n = curs.shape[0]
	curs2 = zeros((n-2,n-2),type=Float32)
	curs_a = zeros((n-2),type=Float32)
	curs_b = zeros((n-2),type=Float32)
	i2 = 0
	for i in range(n):
		if i!=a and i!=b:
			j2 = 0
			for j in range(n):
				if j!=a and j!=b:
					curs2[i2,j2] = curs[i,j]
					j2 += 1
			curs_a[i2] = -curs[i,a]
			curs_b[i2] = -curs[i,b]
			i2+= 1
	v = la.solve_linear_equations(curs2,curs_a)
	g = dot(curs_b,v)-curs[a,b]
	return g

def conductances(curs):
	"""This returns a matrix of conductances between every pair of nodes,
	given the matrix curs of currents as returned by currents().
	
	So for gs=conductances(curs), gs[a][b] is the same value as would be
	returned by conductance(curs,a,b).
	
	TO ADD: right now, this just repeatedly calls conductance(), which is
	much much slower than just inverting the matrix once and multiplying
	thereafter, but that's harder than it sounds... (If even possible?)"""
	n = curs.shape[0]
	gs = []
	for i in range(n):
		gi = []
		for j in range(i):
			gi.append(conductance(curs,i,j))
		gs.append(gi)
	for i in range(n):
		gi = gs[i]
		gi.append(None)
		for j in range(i+1,n):
			gi.append(gs[j][i])
	return gs

def total(gs,i):
	"""Given a matrix gs of conductances, as returned by conductances(),
	this returns the sum of conductances over a given index i, which is
	sort of a measure of the 'total conductance' of the node at i, or the
	extent to which the node at i is 'connected' to the rest of the graph."""
	return sum(gs[i][0:i])+sum(gs[i][i+1:])

def totals(gs):
	"""Returns a list of totals, i.e. totals(gs)[i] is equivalent to
	total(gs,i)."""
	ts = []
	for i in range(n):
		ts.append(totals(gs,i))
